// Generated by CoffeeScript 1.10.0
(function() {
  var Algorithms, Envelope, Protocol, builder, crypto, et, pb, promise, protocol;

  Algorithms = {
    0: 'plaintext',
    1: 'aes',
    2: 'triplesec'
  };

  promise = typeof Promise !== "undefined" && Promise !== null ? Promise : require('es6-promise').Promise;

  crypto = require('crypto');

  et = require('ecc-tools');

  pb = require('protobufjs');

  protocol = 'package protocol;\n\nmessage Envelope {\n  required Cipher cipher = 1;\n  required bytes checksum = 2;\n  optional bytes signature = 3;\n\n  message Cipher {\n    enum Algorithm {\n      PLAINTEXT = 0;\n      plaintext = 0;\n      AES = 1;\n      aes = 1;\n      TRIPLESEC = 2;\n      triplesec = 2;\n    }\n\n    required bytes ciphertext = 1;\n    required Algorithm algorithm = 2;\n\n    optional bytes iv = 3;\n    optional bytes mac = 4;\n    optional bytes ephemeralKey = 5;\n\n    optional bytes to = 6;\n    optional bytes from = 7;\n  }\n}';

  builder = pb.loadProto(protocol);

  Protocol = builder.build('protocol');

  Envelope = (function() {
    function Envelope(args) {
      var base, ref, ref1, ref2, ref3, ref4, ref5, ref6;
      if (args == null) {
        args = {};
      }
      if (!(this instanceof Envelope)) {
        return new Envelope(args);
      }
      this._to = {};
      this._from = {};
      if (this._decode = args.decode) {
        switch (args.as) {
          case 'hex':
            this._buffer = Protocol.Envelope.decodeHex(this._decode);
            break;
          case 'json':
            if (typeof this._decode !== 'string') {
              this._decode = et.stringify(this._decode);
            }
            this._buffer = Protocol.Envelope.decodeJSON(this._decode);
            break;
          default:
            this._buffer = Protocol.Envelope.decode((ref = typeof (base = this._decode).toBuffer === "function" ? base.toBuffer() : void 0) != null ? ref : this._decode);
        }
        this._checksum = this._buffer.checksum;
        this._signature = this._buffer.signature;
        this._cipher = this._buffer.cipher;
        this._to["public"] = (ref1 = (ref2 = this._cipher.to) != null ? typeof ref2.toBuffer === "function" ? ref2.toBuffer() : void 0 : void 0) != null ? ref1 : this._cipher.to;
        this._from["public"] = (ref3 = (ref4 = this._cipher.from) != null ? typeof ref4.toBuffer === "function" ? ref4.toBuffer() : void 0 : void 0) != null ? ref3 : this._cipher.from;
        this._algorithm = Algorithms[this._cipher.algorithm];
        this._seal = promise.resolve(this);
      } else {
        this._to["public"] = args.to;
        this._from["private"] = (ref5 = args.from) != null ? ref5 : et.privateKey();
        this._from["public"] = et.publicKey(this._from["private"]);
        this._data = args.send;
        this._algorithm = (ref6 = args.using) != null ? ref6 : 'aes';
        if (this._to["public"] && this._algorithm !== 'plaintext') {
          this._encryptor = et.encrypt(this._data, this._to["public"], this._algorithm);
        } else if (this._data) {
          this._algorithm = 'plaintext';
          this._encryptor = promise.resolve({
            ciphertext: new Buffer(et.stringify(this._data))
          });
        }
      }
    }

    Envelope.prototype.seal = function() {
      return this._seal != null ? this._seal : this._seal = this._encryptor.then((function(_this) {
        return function(cipher) {
          if (_this._to["public"] != null) {
            cipher.to = et.publicKeyConvert(_this._to["public"], true);
          }
          if (_this._from["public"]) {
            cipher.from = et.publicKeyConvert(_this._from["public"], true);
          }
          cipher.algorithm = _this._algorithm;
          _this._cipher = new Protocol.Envelope.Cipher(cipher);
          return _this._checksum = et.checksum(_this._cipher.encodeJSON());
        };
      })(this)).then((function(_this) {
        return function(checksum) {
          return et.sign(checksum, _this._from["private"]);
        };
      })(this)).then((function(_this) {
        return function(signature) {
          return _this._signature = signature;
        };
      })(this)).then((function(_this) {
        return function() {
          _this._buffer = new Protocol.Envelope({
            cipher: _this._cipher,
            checksum: _this._checksum,
            signature: _this._signature
          });
          return _this;
        };
      })(this));
    };

    Envelope.prototype.verify = function() {
      return this.seal().then((function(_this) {
        return function(e) {
          var base, checksum, ref;
          checksum = new Buffer((ref = typeof (base = e._checksum).toBuffer === "function" ? base.toBuffer() : void 0) != null ? ref : e._checksum);
          return et.verify(checksum, e._from["public"], e._signature) && checksum.equals(et.checksum(e._cipher.encodeJSON()));
        };
      })(this));
    };

    Envelope.prototype.open = function(key) {
      return this.seal().then((function(_this) {
        return function(e) {
          return _this.verify().then(function(valid) {
            var base, ciphertext, ref;
            if (!valid) {
              return false;
            }
            if (_this._algorithm === 'plaintext') {
              ciphertext = new Buffer((ref = typeof (base = e._cipher.ciphertext).toBuffer === "function" ? base.toBuffer() : void 0) != null ? ref : e._cipher.ciphertext);
              _this._decryptor = promise.resolve(JSON.parse(ciphertext));
            } else {
              _this._decryptor = et.decrypt(e._cipher, key, e._algorithm);
            }
            return _this._decryptor.then(function(plaintext) {
              var from, ref1, ref2, ref3, ref4, result, to;
              to = (ref1 = (ref2 = e._to["public"]) != null ? typeof ref2.toBuffer === "function" ? ref2.toBuffer() : void 0 : void 0) != null ? ref1 : e._to["public"];
              from = (ref3 = (ref4 = e._from["public"]) != null ? typeof ref4.toBuffer === "function" ? ref4.toBuffer() : void 0 : void 0) != null ? ref3 : e._from["public"];
              result = {
                data: plaintext
              };
              if (to != null) {
                result.to = new Buffer(to);
              }
              if (from != null) {
                result.from = new Buffer(from);
              }
              return result;
            });
          });
        };
      })(this));
    };

    Envelope.prototype.encode = function(encoding) {
      return this.seal().then((function(_this) {
        return function(e) {
          var buffer, ref;
          switch (encoding) {
            case 'hex':
              return e._buffer.encodeHex();
            case 'json':
              return e._buffer.encodeJSON();
            case 'base64':
              return e._buffer.encode64();
            default:
              buffer = e._buffer.encode();
              return new Buffer((ref = buffer.toBuffer()) != null ? ref : buffer);
          }
        };
      })(this));
    };

    return Envelope;

  })();

  Envelope.et = et;

  Envelope.Protocol = Protocol;

  Envelope.encode = function(buffer) {
    var ref;
    return et.bs58check.encode((ref = typeof buffer.toBuffer === "function" ? buffer.toBuffer() : void 0) != null ? ref : buffer);
  };

  Envelope.decode = function(buffer) {
    var ref;
    return et.bs58check.decode((ref = typeof buffer.toBuffer === "function" ? buffer.toBuffer() : void 0) != null ? ref : buffer);
  };

  module.exports = Envelope;

}).call(this);
